# -*- coding: utf-8 -*-
from Plugins.Bp.geminimain.gLocale import _
from Plugins.Bp.geminimain.gTools import cleanexit, pathExists, gPluginPath, getXMLFile, XML_SERVER
from Plugins.Bp.geminimain.plugin import cgevent
from Plugins.Bp.geminimain.WidgetVariableList import WidgetVariableList
from Plugins.Bp.geminimain.ShowFileList import ShowFileList
from Plugins.Bp.geminimain.WidgetConfig import gConfigWindow
from telnetlib import Telnet

from Screens.ChoiceBox import ChoiceBox
from Tools.XMLTools import stringToXML

from Components.config import config, NoSave, getConfigListEntry, ConfigText, ConfigInteger

FAVORITENFILE = "/etc/enigma2/gemini_cccaminfo.xml"

CC_NAME = 0
CC_URL = 1
CC_PORT = 2
CC_USER = 3
CC_PASS = 4

def createName(z):
	if z[CC_USER] == 0:
		z[C_USER] = "none"
	if z[CC_PASS] == 0:
		z[CC_PASS] = "none"
	return z[CC_NAME] + "\n" + _("Server") + ": " + z[CC_URL] + ":" + z[CC_PORT] + "\n" + _("User") + ": " + z[CC_USER] + "\t" + _("Password") + ": " + z[CC_PASS]

def getStartXML():
	if pathExists(FAVORITENFILE):
		return FAVORITENFILE
	return gPluginPath + "geminicccaminfo/gemini_cccaminfo.xml"

#----------------------------------------------------------------------------------------

class CCcaminfo(WidgetVariableList):
	def __init__(self, session, name=None):
		WidgetVariableList.__init__ (self, session, name or _("CCcam-Info"), icon = "geminicccaminfo/icon_cccaminfo.png", LinesProEntry=3, NameRed=_("Delete"), NameGreen=_("Add"), NameBlue=_("Edit"))
		self.skinName = [self.__class__.__name__,"VariableList3Button_" + self.__class__.__name__,"VariableList3Button"]

		self.newconf=False
		self.onLayoutFinish.append(self.__LoadXML)

	def __LoadXML(self):
		self.hideLabel()
		list = []
		for x in getXMLFile(self, XML_SERVER, getStartXML()):
			if x[CC_NAME]!=0 and x[CC_URL]!=0 and x[CC_PORT]!=0:
				list.append((createName(x),None,x))
		self.SetList(list)
		
	def KeyRed(self):
		val = self.getCurrentEntry()
		if val:
			self.RemoveEntry(self.GetIndex())
			self.newconf=True

	def KeyOk(self):
		sel = self.getCurrentEntry()
		if sel:
			self.session.open(CCCamEntryInfo, sel[2])

	def KeyBlue(self):
		sel = self.getCurrentEntry()
		if sel:
			self.session.openWithCallback(self.__Callbackedit, CCCamEntryEdit, sel[2])
	
	def KeyGreen(self):
		self.session.openWithCallback(self.__Callbackappend, CCCamEntryEdit, ("CCcam new", "localhost", 16000, "none", "none"))

	def __Callbackedit(self, res = False):
		if res:
			self.editEntry(self.GetIndex(),res)
			self.newconf=True
			
	def __Callbackappend(self, res = False):
		if res:
			self.addEntry(res)
			self.newconf=True

	def __writeXML(self):
		try:
			wfile = open(FAVORITENFILE, 'w')
			wfile.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>\n\n")
			wfile.write("<list>\n")
			for x in self.GetList():
				wfile.write("\t<telnetinfo>\n")
				wfile.write("\t\t<name>%s</name>\n" %(stringToXML(x[2][CC_NAME])))
				wfile.write("\t\t<url>%s</url>\n" %(stringToXML(x[2][CC_URL])))
				wfile.write("\t\t<port>%s</port>\n" %(stringToXML(x[2][CC_PORT])))
				wfile.write("\t\t<user>%s</user>\n" %(stringToXML(x[2][CC_USER])))
				wfile.write("\t\t<pass>%s</pass>\n" %(stringToXML(x[2][CC_PASS])))
				wfile.write("\t</telnetinfo>\n")
			wfile.write("</list>\n")
			wfile.close()
		except:
			pass

	def KeyExit(self):
		if self.newconf:
			self.__writeXML()
		self.close(None)
		cleanexit(__name__)

#--------------------------------------------------------------------------------------

class CCCamEntryEdit(gConfigWindow):
	def __init__(self, session, cccamlist):
		list = []
		config.bp.name = NoSave(ConfigText(cccamlist[CC_NAME], fixed_size=False))
		config.bp.server = NoSave(ConfigText(cccamlist[CC_URL], fixed_size=False))
		config.bp.port = NoSave(ConfigInteger(default=int(cccamlist[CC_PORT]), limits=(0, 70000)))
		config.bp.user = NoSave(ConfigText(cccamlist[CC_USER], fixed_size=False))
		config.bp.passwd = NoSave(ConfigText(cccamlist[CC_PASS], fixed_size=False))
		
		list.append(getConfigListEntry(_("Name") + ":", config.bp.name))
		list.append(getConfigListEntry(_("Server")+ ":", config.bp.server))
		list.append(getConfigListEntry(_("Port") + " (TELNETINFO LISTEN PORT):", config.bp.port))
		list.append(getConfigListEntry(_("User") + " (TELNETINFO USERNAME):", config.bp.user))
		list.append(getConfigListEntry(_("Password") + " (TELNETINFO PASSWORD):", config.bp.passwd))
		gConfigWindow.__init__(self, session, title=_("Use Telnetinfo Parameters !!!"), icon="geminicccaminfo/icon_cccaminfo.png",list=list)
	
	def KeyGreen(self):
		list = ["","","","",""]
		list[CC_NAME] = config.bp.name.value
		list[CC_URL] = config.bp.server.value
		list[CC_PORT] = str(config.bp.port.value)
		list[CC_USER] = config.bp.user.value
		list[CC_PASS] = config.bp.passwd.value
		self.close((createName(list),None,list))
		
	def KeyExit(self):
		self.close()

#--------------------------------------------------------------------------------------
class CCCamEntryInfo(WidgetVariableList):
	def __init__(self, session, entry):
		list = []
		list.append((_("About Cccam"), None, "info"))
		list.append((_("Active client(s)"), None, "activeclients"))
		list.append((_("Connected client(s)"), None, "clients"))
		list.append((_("Server connections"), None, "servers"))
		list.append((_("Available shares"), None, "shares",entry))
		list.append((_("Available providers"), None, "providers"))
		list.append((_("Entitlements"), None, "entitlements"))
		
		self.__entry = entry
		WidgetVariableList.__init__ (self, session, entry[CC_NAME], icon = "geminicccaminfo/icon_cccaminfo.png",list=list)
		self.skinName = [self.__class__.__name__,"VariableListNoButton_" + self.__class__.__name__,"VariableListNoButton"]
		
	def KeyExit(self):
		self.close()
		
	def KeyOk(self):
		sel = self.getCurrentEntry()
		if sel:
			what = sel[2]
			try:
				t = Telnet(self.__entry[CC_URL], self.__entry[CC_PORT], 8)
				val=None
				if t:
					if self.__entry[CC_USER] and self.__entry[CC_USER]!="none" and self.__entry[CC_PASS] and self.__entry[CC_PASS] != "none":
						t.read_until(b"Username: ",8)
						t.write(self.__entry[CC_USER] + b"\n")
						t.read_until(b"Password: ",8)
						t.write(self.__entry[CC_PASS] + b"\n")
						t.write(what + b"\n")
						val = t.read_all()
					else:
						t.write(what + b"\n")
						val = t.read_all()
				
				if val:
					list = val.split('\n')
					if len(list)>2:
						del list[0]
						self.session.open(ShowFileList, textlist=list)
						return
					
			except Exception, e:
				print "[%s] <%s>"%(__name__,e)
			self.SetLabel(_("No data found."))

