# -*- coding: utf-8 -*-
from gLocale import _
from gTools import cleanexit, dayStrSmall, gLoadPixmap
from plugin import cgevent

from enigma import eEPGCache, eServiceReference

from WidgetConfig import gConfigWindow, gConfigSelection, CONFIGSEPERATOR
from Components.config import config, NoSave, getConfigListEntry, ConfigText

from Screens.Screen import Screen
from Screens.MessageBox import MessageBox
from Screens.EventView import EventViewSimple
from Components.Label import Label
from Components.ActionMap import ActionMap
from Components.Sources.List import List
from Components.Sources.StaticText import StaticText
from ServiceReference import ServiceReference

from time import localtime

class EPGSearch(gConfigWindow):
	def __init__(self, session):
		gConfigWindow.__init__ (self, session, _("EPG Search"), greenbutton=_("Start"), icon="icon_epgsearch.png")
		self.selected=0
		config.bp.sin = NoSave(gConfigSelection(default=cgevent.getConfig("EPGsearch_in", 0), choices=[(0, _("Title")), (1, _("Title"))]))
		config.bp.sstr= NoSave(ConfigText(default=cgevent.getConfig("EPGsearch_string",_("what")), fixed_size=False))
		self.onLayoutFinish.append(self.__fillListe)

	def __fillListe(self):
		list = []
		list.append(getConfigListEntry(_("search in") + ":", config.bp.sin))
		list.append(getConfigListEntry(_("Text") + ":", config.bp.sstr))
		self.SetConfList(list)
		self.SetIndex(1)
		
	def SelectionChangedNow(self):
		gConfigWindow.SelectionChangedNow(self)
		sel = self.getCurrendValue()
		if sel and sel == config.bp.sstr:
			self.setRB(_("current"))
			self.selected=1
		else:
			self.setRB()
			self.selected=0
			
	def KeyRed(self):
		if self.selected==1:
			service = self.session.nav.getCurrentService()
			if service:
				info = service.info()
				if info:
					event = info.getEvent(0)
					if event:
						config.bp.sstr.value = event.getEventName()
			self.__fillListe()
		
	def KeyGreen(self):
		list = None
		sstr=config.bp.sstr.value
		cgevent.setConfig("EPGsearch_string", sstr)
		cgevent.setConfig("EPGsearch_in", config.bp.sin.value)
		
		try:
			tmpstr = sstr
			tmpstr = tmpstr.decode('UTF-8', 'replace').encode('ISO8859-15', 'replace')
			list = eEPGCache.getInstance().search(('RIBND', 100, eEPGCache.PARTIAL_TITLE_SEARCH, tmpstr, eEPGCache.NO_CASE_CHECK)) or []
		except:
			pass

		if list and len(list):
			list.sort(key=lambda x: x[2])
			self.SetIndex(0)#wegen VirtualKeyboard
			self.session.open(EPGSearchView,list,sstr)
		else:
			self.session.open(MessageBox, (_("'%s' not found in EPG!") %(sstr)), MessageBox.TYPE_INFO, 5)

	def KeyExit(self):
		self.close()
		cleanexit(__name__)

#---------------------------------------------------------------------------------

class EPGSearchView(Screen):
	def __init__(self, session, list, sstr):
		Screen.__init__(self, session)
		self.skinName = [self.__class__.__name__+"_HD",self.__class__.__name__+"_XD",self.__class__.__name__+"_SD"]
		
		self.__epgliste=list
		self.__sstr = sstr

		self["actions"] = ActionMap(["gActions"],
		{
			"exit": self.KeyExit,
			"ok": self.KeyOk,

		}, -1)
		
		self["menu"] = List([])
		self.onLayoutFinish.append(self.__fillList)
		
	def __checkTimer(self, refstr, eventId):
		for x in self.session.nav.RecordTimer.timer_list:
			if x.service_ref.ref.toString() == refstr:
				if x.eit == eventId:
					return True
		return False

	def __fillList(self):
		index = self["menu"].getIndex()
		self.setTitle(_("EPG Search") + " '" + self.__sstr + "'")
		list=[]
		for x in self.__epgliste:
			pix = None
			if self.__checkTimer(x[0],x[1]):
				pix = gLoadPixmap("epgclock.png")
			t = localtime(x[2])
			timestr = "%02d.%02d, %02d:%02d"%(t[2],t[1],t[3],t[4])
			list.append((dayStrSmall[t[6]], timestr, pix, x[3], x))

		self["menu"].setList(list)
		if index>0:
			self["menu"].setIndex(index)
		
	def KeyOk(self):
		cur = self["menu"].getCurrent()
		if cur and cur[4]:
			event = eEPGCache.getInstance().lookupEventId(eServiceReference(cur[4][0]), cur[4][1])
			if event:
				self.session.openWithCallback(self.__fillList, EventViewSimple, event, ServiceReference(cur[4][0]))#Event, Ref

	def KeyExit(self):
		self.close()
