# -*- coding: utf-8 -*-
from gLocale import _
from enigma import ePoint, quitMainloop
from gTools import cleanexit, pathExists, pathRemove, BoxType, getInstalledLang, setInstalledLang, \
	gLoadPixmap, checkCacheDirs, getColorPic, VDesktopfound, GeminiCacheDirs, DEFAULTGEMINICACHEDIR
from plugin import cgevent, SETGENERAL, ORBITAL_POS_SHOW, CHANNEL_NR_SHOW
from lib import libgeminimain

from WidgetVariableList import WidgetVariableList
from WidgetConfig import gConfigWindow, gConfigOnOff, gConfigSelection, CONFIGSEPERATOR, gConfigDirectory
from Components.config import config, NoSave, getConfigListEntry, ConfigText, ConfigInteger

from enigma import eEPGCache
from Screens.MessageBox import MessageBox
from Screens.Screen import Screen
from Components.Console import Console
from Components.ActionMap import ActionMap
from Components.Label import Label


class confGeneral(gConfigWindow ):
	EXTRAPLUGINFILE = "/etc/opkg/gemini-extraPlugins-feed.conf"
	
	def __init__(self, session, name=None):
		
		self.Console = Console()

		geminiconf = libgeminimain.getGeminiConf()
		self.__selected=0
		self.__OKstate = cgevent.getConfig("OkButton",0)
		self.__DisplayAsClock = cgevent.getConfig("DisplayAsClock",0)

		menu = [(0, _("off")),(1, _("blinking")), (2, _("on"))]
		config.bp.bgevent = NoSave(gConfigSelection(default=cgevent.getConfig("bg_event",1), choices=menu))
		config.bp.orbital = NoSave(gConfigOnOff(default=bool(geminiconf & ORBITAL_POS_SHOW)))
		config.bp.channelnr = NoSave(gConfigOnOff(default=bool(geminiconf & CHANNEL_NR_SHOW)))
		config.bp.position = NoSave(gConfigSelection(choices=[_("View") + "/" + _("Choose")]))
		config.bp.picon = NoSave(gConfigDirectory(default=cgevent.getConfig("location_picon","/usr/share/enigma2/picon"), visible_width=35))
		menu = [(0, _("off")),(1, _("Show '%s' in the infobar") %"UP"), (2, _("Show Icon")), (3, _("Show Message"))]
		config.bp.search = NoSave(gConfigSelection(default=cgevent.getConfig("checkUpgrade",0), choices=menu))
		config.bp.extra = NoSave(gConfigOnOff(default=int(pathExists(self.EXTRAPLUGINFILE))))
		config.bp.cache = NoSave(gConfigDirectory(default=cgevent.getConfig("geminicache",DEFAULTGEMINICACHEDIR[:-1]), visible_width=35))
		config.bp.opkg = NoSave(gConfigOnOff(default=cgevent.getConfig("opkg_conf",0)))
		config.bp.epgcache = NoSave(gConfigDirectory(default=cgevent.getConfig("epgCacheDir","/media/hdd"), visible_width=35))
		menu = [(0, _("Function disable")),(1, _("Function enable (off)")), (2, _("Function enable (on)"))]
		config.bp.okbutton = NoSave(gConfigSelection(default=self.__OKstate, choices=menu))
		config.bp.lang = NoSave(gConfigSelection(choices=[_("Press OK for details")]))
		config.bp.mediascan = NoSave(gConfigOnOff(default=cgevent.getConfig("media_scan",1)))
		config.bp.display = NoSave(gConfigOnOff(default=self.__DisplayAsClock))
		
		#ChannelSelection
		self.__chsmenu=[]
		count=0
		defaultheight = cgevent.getConfig("ChannelSelHeight",-1)
		
		for x in cgevent.ChannelSelectionList:
			if int(x[0]) == cgevent.ChannelSelectionHeight:
				self.__chsmenu.append((count,"%s (%s)" %(x[0],_("Default"))))
				if defaultheight==-1:
					defaultheight=count
			else:
				self.__chsmenu.append((count,x[0]))
			count += 1
		if len(self.__chsmenu):
			config.bp.channelselection = NoSave(gConfigSelection(default=defaultheight, choices=self.__chsmenu))

		list = []
		list.append(getConfigListEntry(_("Location for %s") %"Picons", config.bp.picon))
		list.append(getConfigListEntry(_("Location for %s") %"Gemini-Cache", config.bp.cache))
		list.append(getConfigListEntry(_("Location for %s") %"EPG-Cache", config.bp.epgcache))
		list.append(CONFIGSEPERATOR())
		list.append(getConfigListEntry(_("Automatically scan for Media-Infos?"), config.bp.mediascan))
		list.append(CONFIGSEPERATOR())
		list.append(getConfigListEntry(_("Display always as clock"), config.bp.display))
		list.append(getConfigListEntry(_("OK Button for show Infobar"), config.bp.okbutton))
		list.append(getConfigListEntry(_("Show background events ?"), config.bp.bgevent))
		list.append(getConfigListEntry(_("Show Orbitalposition") + " " + _("(only Gemini-Skins)"), config.bp.orbital))
		list.append(getConfigListEntry(_("Show Channelnumber") + " " + _("(only Gemini-Skins)"), config.bp.channelnr))
		list.append(getConfigListEntry(_("Position of the Infobar"), config.bp.position))
		if len(self.__chsmenu):
			list.append(getConfigListEntry(_("Height of entries in the channel list"), config.bp.channelselection))
		list.append(CONFIGSEPERATOR())
		list.append(getConfigListEntry(_("Looking for daily updates"), config.bp.search))
		list.append(getConfigListEntry(_("only 'Root' for OPKG locations"), config.bp.opkg))
		list.append(getConfigListEntry(_("use extra Plugins ?"), config.bp.extra))
		list.append(getConfigListEntry(_("Languages remove ?"), config.bp.lang))
				
		if VDesktopfound:
			config.bp.vdesktop = NoSave(gConfigOnOff(default=cgevent.getConfig("vdesktop_conf",1)))
			list.append(CONFIGSEPERATOR())
			list.append(getConfigListEntry(_("Virtual Desktop"), config.bp.vdesktop))

		gConfigWindow.__init__ (self, session, name or _("General"), list=list, icon="icon_general.png")

	def SelectionChangedNow(self):
		gConfigWindow.SelectionChangedNow(self)
		self.setRB()
		self.__selected=0
		if self.getCurrendValue() == config.bp.cache:
			self.setRB(_("Delete"))
			self.__selected=1
		
	def KeyLeft(self):
		if self.getCurrendValue() == config.bp.position:
			self.session.open(InfobarPreview, False)
		else:
			gConfigWindow.KeyLeft(self)

	def KeyRight(self):
		if self.getCurrendValue() == config.bp.position:
			self.session.open(InfobarPreview, True)
		else:
			gConfigWindow.KeyRight(self)
			
	def KeyOk(self):
		if self.getCurrendValue() == config.bp.lang:
			self.session.open(LanguageSetup)
		else:
			gConfigWindow.KeyOk(self)

	def KeyGreen(self):
		cgevent.setConfig("media_scan", config.bp.mediascan.value)
		cgevent.setConfig("DisplayAsClock", config.bp.display.value)
		#ChannelSelection
		if len(self.__chsmenu):
			cgevent.setConfig("ChannelSelHeight",config.bp.channelselection.value)
			cgevent.setChannelSelection(config.bp.channelselection.value)
		#vdesktop
		if VDesktopfound:
			cgevent.setConfig("vdesktop_conf",config.bp.vdesktop.value)
		
		libgeminimain.gFunktion(SETGENERAL, (config.bp.bgevent.value, config.bp.picon.value, config.bp.search.value, config.bp.orbital.value, config.bp.channelnr.value, config.bp.opkg.value, config.bp.cache.value))
		
		#Cache
		checkCacheDirs()
		#ExtraPlugins
		if config.bp.extra.value == 1 and pathExists(self.EXTRAPLUGINFILE)==False:
			try:
				wfile = open(self.EXTRAPLUGINFILE, 'w')
				if BoxType == "dm800":
					wfile.write("src/gz gemini-extraPlugins http://download.blue-panel.com/gemini32/extraPlugins-nf\n")
				else:
					wfile.write("src/gz gemini-extraPlugins http://download.blue-panel.com/gemini32/extraPlugins\n")
				wfile.close()
			except:
				pass
		elif config.bp.extra.value == 0:
			pathRemove(self.EXTRAPLUGINFILE)

		#EPG
		epgcachefile = config.bp.epgcache.value + "/epg.dat"
		cgevent.setConfig("epgCacheDir", config.bp.epgcache.value)
		config.misc.epgcache_filename = ConfigText(default = epgcachefile)
		eEPGCache.getInstance().setCacheFile(epgcachefile)
		#OK Button
		cgevent.setConfig("OkButton", config.bp.okbutton.value)
		cgevent.setOkButtonConfig()
		
		#muss restart
		restart = False
		if self.__OKstate == 0 and config.bp.okbutton.value != 0:
			restart = True
		if self.__OKstate > 0 and config.bp.okbutton.value == 0:
			restart = True
		if self.__DisplayAsClock != config.bp.display.value:
			restart = True
	
		if restart:
			self.session.openWithCallback(self.__restartOK, MessageBox, _("Do you want to restart Enigma2?"), MessageBox.TYPE_YESNO)
		else:
			self.KeyExit()
	
	def __restartOK(self, result):
		if result:
			quitMainloop(3)
		else:
			self.KeyExit()

	def KeyRed(self):
		if self.__selected==1:
			cmd=""
			for x in GeminiCacheDirs:
				if cmd!="":
					cmd += " & "
				cmd += "rm -r '%s/%s/'*" %(config.bp.cache.value,x)
			self.Console.ePopen(cmd, self.__FinishedConsole)
			
	def __FinishedConsole(self, result, retval, extra_args = None):
		checkCacheDirs()
		self.session.open(MessageBox, _("done..."), MessageBox.TYPE_INFO, timeout=3)

	def KeyExit(self):
		self.close()
		cleanexit(__name__)

#--------------------------------------------------------------------------------------

class InfobarPreview(Screen):
	def __init__(self, session, activ=False):
		self.activ = activ
		self.size = cgevent.INFOBAR_SIZE
		self.pos = cgevent.INFOBAR_POSITION
		self.offset_x = cgevent.getConfig("infobar_offset_x",0)
		self.offset_y = cgevent.getConfig("infobar_offset_y",0)

		self.skin = "<screen position=\"" + str(self.pos.x()+self.offset_x) + "," + str(self.pos.y()+self.offset_y) + "\" size=\"" + str(self.size.width()) + "," + str(self.size.height()) + "\" flags=\"wfNoBorder\" > \
				<widget name=\"label\" position=\"5,5\" size=\"" + str(self.size.width()-10) + "," + str(self.size.height()-10) + "\" font=\"Regular;19\" halign=\"center\" valign=\"center\" /> </screen>"

		Screen.__init__(self, session)

		self["actions"] = ActionMap(["gActions"],
		{
			"ok": self.KeyOK,
			"up": self.KeyUp,
			"down": self.KeyDown,
			"left": self.KeyLeft,
			"right": self.KeyRight,
			"exit": self.KeyExit
		}, -1)

		self["label"] = Label(_("Position") + " = " + str(self.pos.x() + self.offset_x) + "," + str(self.pos.y() + self.offset_y))
		self.onLayoutFinish.append(self.__InfobarMove)

	def __InfobarMove(self):
		if self.activ:
			self.instance.move(ePoint(self.pos.x() + self.offset_x, self.pos.y() + self.offset_y))

			self["label"].setText(_("Press (left, right) or (up, down)\n") + _("Press OK to save settings") + "\n" + _("Original Position") + " = " + str(self.pos.x()) + "," + str(self.pos.y()) + "\n" + _("New Position") + " = " + str(self.pos.x() + self.offset_x) + "," + str(self.pos.y() + self.offset_y))

	def KeyUp(self):
		self.offset_y -= 1
		self.__InfobarMove()
		
	def KeyDown(self):
		self.offset_y += 1
		self.__InfobarMove()
		
	def KeyRight(self):
		self.offset_x += 1
		self.__InfobarMove()

	def KeyLeft(self):
		self.offset_x -= 1
		self.__InfobarMove()
		
	def KeyOK(self):
		if self.activ:
			cgevent.setConfig("infobar_offset_x", self.offset_x)
			cgevent.setConfig("infobar_offset_y", self.offset_y)
			cgevent.setInfobarOffset()
		self.KeyExit()
		
	def KeyExit(self):
		self.close()

#--------------------------------------------------------------------------------------

from Components.Language import language
from Components.language_cache import LANG_TEXT


def _cached(x):
	return LANG_TEXT.get(config.osd.language.value, {}).get(x, "")

class LanguageSetup(WidgetVariableList):
	def __init__(self, session):
		WidgetVariableList.__init__ (self, session, title=_("Languages remove ?"), ListIcon=(35,25))
		self.skinName = [self.__class__.__name__,"VariableListNoButton_" + self.__class__.__name__,"VariableListNoButton"]
		self.onLayoutFinish.append(self.__fillListe)
		
			
	def __foundLang(self,val):
		for x in getInstalledLang():
			if x == val:
				return True
		return False
	
	def __fillListe(self):
		list = []
		for x in language.getLanguageList():
			found = self.__foundLang(x[1][2].lower())
			list.append((_cached("%s_%s" % x[1][1:3]), getColorPic(found), x, found, found))
		self.SetList(list)
		
	def KeyOk(self):
		sel = self.getCurrentEntry()
		if sel:
			if sel[3] and sel[2][1][2].lower() != "en":
				remove=True
				if sel[4]:
					remove=False
				entry = (sel[0],getColorPic(remove),sel[2],sel[3],remove)
				self.editEntry(self.GetIndex(),entry)
				
	def KeyExit(self):
		remove=False
		for x in self.GetList():
			if x[3] != x[4]:
				remove=True
				break
		if remove:
			self.session.openWithCallback(self.Exit, MessageBox, _("Do you want to remove languages?"), MessageBox.TYPE_YESNO)
		else:
			self.Exit()
		
	def Exit(self,val=False):
		if val:
			langstr=""
			for x in self.GetList():
				if x[4]:
					langstr += x[2][1][2].lower()+","
			setInstalledLang(langstr)
		self.close()
