# -*- coding: utf-8 -*-
from gLocale import _
from gTools import cleanexit, dayStrSmall
from gweathercore import gwtools
from plugin import cgevent

from Screens.Screen import Screen
from Components.ActionMap import ActionMap
from Components.Sources.StaticText import StaticText
from Components.Sources.CanvasSource import CanvasSource

from skin import parseColor, colorNames
from enigma import getDesktop, gFont, RT_VALIGN_CENTER, RT_HALIGN_CENTER
from time import localtime

class Cweatherstat(Screen):
	def __init__(self, session, cityname, firstdaytemp):
		self.__firstdaytemp=firstdaytemp
		self.__cityname = "%s    %s" %(cityname, _("Temperature"))
		size = getDesktop(0).size()
		width = int(size.width() * 0.9)
		height = int(size.height() * 0.8)
		self.Cwidth = width-10
		self.Cheight = height-10
		
		Cweatherstat.skin = """<screen name="Cweatherstat" position="center,center" size="%(w)d,%(h)d" title="%(city)s">
			<widget source="canvas" render="Canvas" position="5,5" zPosition="1" size="%(mainW)d,%(mainH)d" backgroundColor="#FF000000" transparent="1" alphatest="on"/>
		</screen>""" %{	"w" : width,
				"h" : height,
				"mainH" : self.Cheight,
				"mainW" : self.Cwidth,
				"city" : self.__cityname
				}
		
		Screen.__init__(self, session)

		self["actions"] = ActionMap(["gActions"],
		{
			"exit": self.KeyExit
		}, -1)
		
		self["title"] = StaticText()
		self["canvas"] =  CanvasSource()
		self.onLayoutFinish.append(self.__firstStart)

	def KeyExit(self):
		self.close()
		cleanexit(__name__)
		
	def __firstStart(self):
		self.setTitle(self.__cityname)
		self["title"].setText(self.__cityname)

		templist=[]
		maxt=-2000
		mint=2000

		temp_unit = cgevent.getConfig("gweather_tempunit",1)
		if temp_unit==1:
			tempunit = "\xc2\xb0C"
			offset=15
		elif temp_unit==2:
			tempunit = "\xc2\xb0F"
			offset=30
		else:
			tempunit = "K"
			offset=30
		
		#min max temp ermitteln und alles *10
		count=0
		for x in gwtools.weatherdata:
			if x[gwtools.TIME]==None or x[gwtools.TEMP][0]!="":
				continue
			#print x[gwtools.TEMP]
			temp_a = int(float(x[gwtools.TEMP][1][:-len(tempunit)])*10)
			if temp_a>maxt:
				maxt=temp_a
			if temp_a<mint:
				mint=temp_a
			temp_b = int(float(x[gwtools.TEMP][2][:-len(tempunit)])*10)
			if temp_b>maxt:
				maxt=temp_b
			if temp_b<mint:
				mint=temp_b
			
			#Wochentag
			tm = localtime(x[gwtools.TIME])
			if self.__firstdaytemp:
				templist.append((temp_b,temp_a, dayStrSmall[tm.tm_wday]))
			else:
				templist.append((temp_a,temp_b, dayStrSmall[tm.tm_wday]))
			
			if count==13:
				break
			count+=1

		#Colors
		colorblue=0x0064c7
		if "blue" in colorNames:
			colorblue=colorNames["blue"].argb()
		colorred=0xf23d21
		if "red" in colorNames:
			colorred=colorNames["red"].argb()
		colorfg=0xffffff
		if "foreground" in colorNames:
			colorfg=colorNames["foreground"].argb()
		colorbg=0x25062748
		if "background" in colorNames:
			colorbg=colorNames["background"].argb()
			
		firstcolor=colorblue
		secondcolor=colorred
		if self.__firstdaytemp:
			firstcolor=colorred
			secondcolor=colorblue

		maxt+=offset
		mint-=offset
		nullpos = None
		nulltextW=50
		fontH=25

		cx = ((self.Cwidth-nulltextW) / len(templist) / 2)
		if mint<0:
			m = float(self.Cheight / (maxt+abs(mint)))
		else:
			m = float(self.Cheight / (maxt-mint))
			
		#print "maxt =",maxt,"\tmint =",mint,"\thöhe =",self.Cheight,"\tmulti =",m
		self["canvas"].writeText(0, self.Cheight-fontH,nulltextW,fontH,colorfg,colorbg,gFont("Regular", fontH-6),"%s" %tempunit, RT_HALIGN_CENTER)
		
		if mint<0:
			#Null-Linie
			nullpos = int(self.Cheight-(m * abs(mint)))
			self["canvas"].fill(5, nullpos-1, self.Cwidth, 1, colorfg)#x y weite tiefe
			self["canvas"].writeText(5, nullpos-fontH,nulltextW,fontH,colorfg,colorbg,gFont("Regular", fontH-6),"0", RT_HALIGN_CENTER)
			#Max Line
			maxpos=int(nullpos -(m * (maxt-offset)))
			self["canvas"].fill(5, maxpos, self.Cwidth, 1, colorfg)
			self["canvas"].writeText(5, maxpos-fontH,nulltextW,fontH,colorfg,colorbg,gFont("Regular", fontH-6),"%.1f" %(float(maxt-offset)/10))
			#Min-Line
			minpos=int(nullpos -(m * (mint+offset)))
			self["canvas"].fill(5, minpos, self.Cwidth, 1, colorfg)
			self["canvas"].writeText(5, minpos-fontH,nulltextW,fontH,colorfg,colorbg,gFont("Regular", fontH-6),"%.1f" %(float(mint+offset)/10))
		else:
			#Max Line
			maxpos = int(m*(maxt-maxt+offset))
			self["canvas"].fill(5, maxpos, self.Cwidth, 1, colorfg)
			self["canvas"].writeText(5, maxpos-fontH,nulltextW,fontH,colorfg,colorbg,gFont("Regular", fontH-6),"%.1f" %(float(maxt-offset)/10))
			#Min-Line
			minpos = int(m*(maxt-mint-offset))
			self["canvas"].fill(5, minpos, self.Cwidth, 1, colorfg)
			self["canvas"].writeText(5, minpos-fontH,nulltextW,fontH,colorfg,colorbg,gFont("Regular", fontH-6),"%.1f" %(float(mint+offset)/10))
			
			diff = (maxt-offset - mint+offset) / 2
			mittelpos = int((minpos-maxpos)/2)
			self["canvas"].fill(5, mittelpos, self.Cwidth, 1, colorfg)
			self["canvas"].writeText(5, mittelpos-fontH,nulltextW,fontH,colorfg,colorbg,gFont("Regular", fontH-6),"%.1f" %((float(mint+offset+diff)/10)))
		
		posx=0
		for x in templist:
			#print x
			self["canvas"].fill(posx+nulltextW-1, 0, 1, self.Cheight, colorfg)
			self["canvas"].writeText(posx+nulltextW, self.Cheight-fontH,cx,fontH,colorfg,colorbg,gFont("Regular", fontH-7),"%.1f" %(float(x[0])/10), RT_HALIGN_CENTER)
			self["canvas"].writeText(posx+nulltextW+cx, self.Cheight-fontH,cx,fontH,colorfg,colorbg,gFont("Regular", fontH-7),"%.1f" %(float(x[1])/10), RT_HALIGN_CENTER)
			#Wochentag
			self["canvas"].writeText(posx+nulltextW, 0,cx*2,fontH,colorfg,colorbg,gFont("Regular", fontH-5),x[2], RT_HALIGN_CENTER)
			
			if nullpos:
				#Nachtwerte
				if x[0]<0:
					self["canvas"].fill(posx+nulltextW, nullpos, cx, int(m*abs(x[0])), firstcolor)
				else:
					self["canvas"].fill(posx+nulltextW, int(nullpos -(m * x[0])), cx, int(m*x[0]), firstcolor)
				#Tagwerte
				if x[1]<0:
					self["canvas"].fill(posx+nulltextW+cx, nullpos, cx, int(m*abs(x[1])), secondcolor)
				else:
					self["canvas"].fill(posx+nulltextW+cx, int(nullpos -(m * x[1])), cx, int(m*x[1]), secondcolor)
			else:
				#Nachtwerte
				pos0 = int(m*(maxt-x[0]))
				self["canvas"].fill(posx+nulltextW, pos0, cx, self.Cheight-pos0-fontH-offset, firstcolor)
				#Tagwerte
				pos1 = int(m*(maxt-x[1]))
				self["canvas"].fill(posx+nulltextW+cx, pos1, cx, self.Cheight-pos1-fontH-offset, secondcolor)
				
			posx+=cx*2
			
		self["canvas"].flush()
		