# -*- coding: utf-8 -*-
from Plugins.Bp.geminimain.gLocale import _
from Plugins.Bp.geminimain.gTools import cleanexit, pathExists, IP2Config, SMALLUSECHAR
from Plugins.Bp.geminimain.WidgetVariableList import WidgetVariableList
from Plugins.Bp.geminimain.WidgetConfig import gConfigWindow, WidgetInput, CONFIGSEPERATOR, gConfigSelection, gConfigOnOff
from Plugins.Bp.geminimain.plugin import cgevent, dbusNotifier
from Plugins.Bp.geminimain.lib import libgeminimain
from Plugins.Bp.geminimain.InfoHardware import NET_TYPE, NET_IP, NET_ACTIVE, NET_MASK, NET_BCAST, NETDEV_ETH

from Screens.ChoiceBox import ChoiceBox
from Screens.MessageBox import MessageBox
from Tools.BoundFunction import boundFunction
from Components.config import config, NoSave, ConfigText, ConfigInteger, getConfigListEntry, ConfigIP
from enigma import eTimer

from time import strftime, localtime
from os import chmod

#GVPNEXEC="/home/root/bin/gvpn"
GVPNEXEC="/usr/bin/gvpn"
OPENVPNLOG="/var/log/openvpn.log"
OVPNCONFIG="/etc/openvpn/server.conf"
OPENSSLDIR="/etc/ssl/openvpn"
OPENVPNDIR="/etc/openvpn"
OPENVPNSCRIPT="/etc/init.d/openvpn"

F_ERROR=0
F_INFO=1
F_SEND_CERT=2
F_DONE=3
F_OPENVPN_STATUS=4
F_CREATE_CA=5
F_CREATE_CERT_SERVER=6
F_CREATE_CERT_CLIENT=7
F_CREATE_CERT_INFOS=8
F_REMOVE_CERT=9
F_START_CLIENT=10
F_STOP_CLIENT=11
F_CLIENT_STATUS=12

class OpenVPNserver(WidgetVariableList):
	CREATEDHFILE="/tmp/.createDH"

	def __init__(self, session):
		WidgetVariableList.__init__ (self, session, title=_("Openvpn-Server"), LinesProEntry=3, NameGreen=_("Certificates"), icon="geminiopenvpn/icon_openvpn.png", list=[], NameBlue=_("Menu"))
		self.skinName = [self.__class__.__name__, "VariableList3Button_" + self.__class__.__name__, "VariableList3Button"]
		self.__timer = eTimer()
		self.__timer.callback.append(self.__Status)
		self.__timer.start(300)
		self.__timercount=0
		self.__logfilefound=False
		self.__isrunning=False
		dbusNotifier.append(("gvpn", self.__newevent))
		from Plugins.Bp.geminimain.lib import libgtools
		self.__libgtools = libgtools

	def KeyExit(self):
		del self.__timer
		del self.__libgtools
		dbusNotifier.remove(("gvpn", self.__newevent))
		self.close()
		cleanexit(__name__)

	def KeyGreen(self):
		self.session.open(OpenVPNCert)
		
	def KeyBlue(self):
		menu=[]
		if self.__isrunning>0:
			menu.append((_("stop %s")%"openvpn", "stopdaemon"))
		else:
			if pathExists(OVPNCONFIG):
				menu.append((_("start %s")%"openvpn", "startdaemon"))
		menu.append((_("show %s") %"'Routes'", "routes"))
		menu.append(("--", "--"))
		menu.append((_("Setup"), "setup"))
		self.session.openWithCallback(self.__menuCallback, ChoiceBox, list=menu)
		
	def __menuCallback(self, choice):
		if choice == None:
			return
		if choice[1] == "setup":
			self.session.openWithCallback(self.__setupCallback, OVPNsetup, self.__libgtools)
		elif choice[1] == "stopdaemon":
			libgeminimain.sendDbus("gdaemon",0,OPENVPNSCRIPT + " stop")
		elif choice[1] == "startdaemon":
			libgeminimain.sendDbus("gdaemon",0,OPENVPNSCRIPT + " start")
			self.__timer.start(1000,True)
		elif choice[1]=="routes":
			from Plugins.Bp.geminimain.InfoScreens import showRoutes
			self.session.open(showRoutes,self.__libgtools)
			
	def __setupCallback(self, ret):
		if ret:
			if self.__isrunning==0:
				libgeminimain.sendDbus("gdaemon",0,OPENVPNSCRIPT + " start")
			else:
				libgeminimain.sendDbus("gdaemon",0,OPENVPNSCRIPT + " reload")
			self.__timer.start(1000,True)
		
	def KeyRed(self):
		if self.__logfilefound:
			from  Plugins.Bp.geminimain.ShowFileList import ShowFileList
			self.session.open(ShowFileList, OPENVPNLOG, selectlast=True)
		
	def __Status(self):
		pooltime=3
		self.SetRB()
		self.__isrunning = libgeminimain.isRunning("openvpn")

		if self.__isrunning==0:
			if len(self.GetList())>0:
				self.SetList([])
			if pathExists(self.CREATEDHFILE):
				self.SetLabel(_("please wait, create DH-Parameter"))
			else:
				self.SetLabel("openvpn" + " " + _("not running"))
		else:
			if self.__timercount < 1:
				libgeminimain.sendDbus("gdaemon",0,"%s -f %d" %(GVPNEXEC,F_OPENVPN_STATUS))
				self.__timercount=30
			else:
				self.__timercount -= pooltime
				self.SetLabel(_("read new status in %d second") %self.__timercount)
		
			self.__logfilefound = pathExists(OPENVPNLOG)
			if self.__logfilefound:
				self.SetRB(_("Logfile"))

		self.__timer.start(pooltime*1000,True)
		
	def __fillList(self,val):
		list=[]
		try:
			clientlist=[]
			entry=[]
			for x in val.split('\n'):
				entry.append(x)
				if len(entry)==3:
					clientlist.append(entry)
					entry=[]
		
			for x in clientlist:
				tmp = "%s: %s\t%s: %s\n" %(_("Client"),x[0],_("IP"),x[1])
				date = int(x[2])
				if date>0:
					tmp += _("connected since: %s") %strftime("%H:%M  %d.%m.%Y", localtime(date))
				list.append((tmp, None))
		except:
			pass
		self.SetList(list)
		
	def __newevent(self, func, value):
		#print func,"_____________________",value
		if func==F_OPENVPN_STATUS:
			self.__fillList(value)

#----------------------------------------------------------------------------

class OpenVPNCert(WidgetVariableList):
	CERT_NAME=0
	CERT_FILE=1
	CERT_DATE=2
	CERT_PROJ=3
	CERT_ORGA=4
	CERT_EDATE=5
	CERT_TYPE=6
	
	TYPE_ROOT="1"
	TYPE_SERVER="2"
	TYPE_CLIENT="3"
	
	def __init__(self, session):
		WidgetVariableList.__init__ (self, session, title=_("Certificates"), LinesProEntry=2, icon="geminiopenvpn/icon_openvpn.png", list=[], NameGreen=_("Add"))
		self.skinName = [self.__class__.__name__, "VariableList3Button_" + self.__class__.__name__, "VariableList3Button"]
		dbusNotifier.append(("gvpn", self.__newBTevent))
		libgeminimain.sendDbus("gdaemon",0,"%s -f %d" %(GVPNEXEC,F_CREATE_CERT_INFOS))
		self.__lockactiv=False

	def KeyExit(self):
		dbusNotifier.remove(("gvpn", self.__newBTevent))
		self.close()
		
	def KeyGreen(self):
		self.SetLabel()
		self.session.openWithCallback(self.__setupCallback, CertEntry)
		
	def KeyRed(self):
		if self.__lockactiv:
			sel = self.getCurrentEntry()
			if sel and sel[2]:
				self.session.openWithCallback(boundFunction(self.__RemoveClient,sel[2]), MessageBox, _("Certificate '%s' lock really?") %sel[2][self.CERT_NAME], MessageBox.TYPE_YESNO)
			
	def __RemoveClient(self, val, result):
		if result:
			self.session.openWithCallback(boundFunction(self.__passCallback,val), WidgetInput, title=_("password from root certificate"), entryname=_("Password"), limits=SMALLUSECHAR)
	
	def __passCallback(self, val, passwd=None):
		if passwd is not None:
			cmd = "%s -f %d -p '%s' -n '%s'" %(GVPNEXEC,F_REMOVE_CERT,passwd,val[self.CERT_NAME])
			libgeminimain.sendDbus("gdaemon",0,cmd)
		
	def selectionChanged(self):
		sel = self.getCurrentEntry()
		if sel and sel[2]:
			self.SetRB()
			self.__lockactiv=False
			if sel[2][self.CERT_TYPE] == self.TYPE_CLIENT and sel[2][self.CERT_EDATE]=="0":
				self.SetRB(_("lock"))
				self.__lockactiv=True
		
	def __fillList(self,val):
		list=[]
		certlist=[]
		entry=[]
		for x in val.split('\n'):
			entry.append(x)
			if len(entry)==7:
				certlist.append(entry)
				entry=[]
		
		for x in certlist:
			#print x
			tmp = "%s: %s" %(_("Name"),x[self.CERT_NAME])
			if x[self.CERT_ORGA]!="":
				tmp += "\t%s: %s" %(_("Type"), _(x[self.CERT_ORGA]))
			tmp += "\n%s: %s\t" %(_("Project"),x[self.CERT_PROJ])
			date = int(x[self.CERT_DATE])
			expired_date = int(x[self.CERT_EDATE])
			if expired_date>0:
				tmp += _("expired") + ": %s" %strftime("%d.%m.%Y", localtime(expired_date))
			else:
				if date>0:
					tmp += _("valid until: %s") %strftime("%d.%m.%Y", localtime(date))
			list.append((tmp, None, x))
		self.SetList(list)
		
	def __setupCallback(self, val):
		if val is not None:
			libgeminimain.sendDbus("gdaemon",0,val)
		
	def __newBTevent(self, func, value):
		#print func,"_____________________",value
		if func==F_INFO:
			self.SetLabel(_(value))
		elif func==F_ERROR:
			self.SetLabel("%s: %s" %(_("Error"),value))
		elif func==F_SEND_CERT:
			self.SetLabel()
			self.__fillList(value)
		elif func==F_DONE:
			self.SetLabel(_("done..."))
			libgeminimain.sendDbus("gdaemon",0,"%s -f %d" %(GVPNEXEC,F_CREATE_CERT_INFOS))

#----------------------------------------------------------------------------

class CertEntry(gConfigWindow):
	def __init__(self, session):
		gConfigWindow.__init__(self, session)
		
		menu = [(F_CREATE_CA, "root " + _("Certificate")),(F_CREATE_CERT_SERVER, _("Server")), (F_CREATE_CERT_CLIENT, _("Client"))]
		default = F_CREATE_CERT_CLIENT
		if pathExists(OPENSSLDIR+"/"+"Server-cert.pem")==False:
			default = F_CREATE_CERT_SERVER
		if pathExists(OPENSSLDIR+"/"+"vpn-ca.pem")==False:
			default = F_CREATE_CA
			
		config.bp.func = NoSave(gConfigSelection(default=default, choices=menu))
		config.bp.passwd = NoSave(ConfigText(default="foobar", fixed_size=False))
		config.bp.passwd.setUseableChars(SMALLUSECHAR)
		config.bp.days = NoSave(ConfigInteger(default=10, limits=[1,10]))
		config.bp.country = NoSave(ConfigText(default="DE", fixed_size=True))
		config.bp.country.setUseableChars('ABCDEFGHIJKLMNOPQRSTUVWXYZ')
		config.bp.city = NoSave(ConfigText(default="City", fixed_size=False))
		config.bp.city.setUseableChars(SMALLUSECHAR)
		config.bp.project = NoSave(ConfigText(default="GeminiVPN", fixed_size=False))
		config.bp.project.setUseableChars(SMALLUSECHAR)
		config.bp.state = NoSave(ConfigText(default="Sachsen", fixed_size=False))
		config.bp.state.setUseableChars(SMALLUSECHAR)
		config.bp.name = NoSave(ConfigText(default="user", fixed_size=False))
		config.bp.name.setUseableChars(SMALLUSECHAR)
		self.__fillList()
		
	def KeyLeft(self):
		gConfigWindow.KeyLeft(self)
		if self.getCurrendValue() == config.bp.func:
			self.__fillList()

	def KeyRight(self):
		gConfigWindow.KeyRight(self)
		if self.getCurrendValue() == config.bp.func:
			self.__fillList()
		
	def __fillList(self):
		list=[]
		list.append(getConfigListEntry(_("Type") + ":", config.bp.func))
		list.append(CONFIGSEPERATOR())
		if config.bp.func.value == F_CREATE_CA:
			list.append(getConfigListEntry(_("Password") + ":", config.bp.passwd))
		list.append(getConfigListEntry(_("valid for x years") + ":", config.bp.days))
		if config.bp.func.value == F_CREATE_CA:
			list.append(getConfigListEntry(_("Project") + ":", config.bp.project))
			list.append(CONFIGSEPERATOR())
			list.append(getConfigListEntry(_("Country") + ":", config.bp.country))
			list.append(getConfigListEntry(_("City") + ":", config.bp.city))
			list.append(getConfigListEntry(_("State") + ":", config.bp.state))
		if config.bp.func.value == F_CREATE_CERT_CLIENT:
			list.append(getConfigListEntry(_("Name") + ":", config.bp.name))
		self.SetConfList(list)
		
	def KeyGreen(self):
		if config.bp.func.value==F_CREATE_CA:
			cmd = "%s -f %d -p '%s' -d %d -l %s -c '%s' -s '%s' -n '%s'" %(GVPNEXEC, config.bp.func.value, config.bp.passwd.value, config.bp.days.value*365, config.bp.country.value, config.bp.city.value, config.bp.state.value, config.bp.project.value)
			self.close(cmd)
		else:
			self.session.openWithCallback(self.__passCallback, WidgetInput, title=_("password from root certificate"), entryname=_("Password"), limits=SMALLUSECHAR)
			
	def __passCallback(self,passwd=None):
		if passwd is not None:
			cmd = "%s -f %d -p '%s' -d %d" %(GVPNEXEC, config.bp.func.value, passwd, config.bp.days.value*365)
			if config.bp.func.value == F_CREATE_CERT_CLIENT:
				cmd += " -n '%s'" %config.bp.name.value
			self.close(cmd)
	
	def KeyExit(self):
		self.close(None)

class OVPNsetup(gConfigWindow):
	OVPNUPSCRIPT=OPENVPNDIR + "/up.sh"
	OVPNDOWNSCRIPT=OPENVPNDIR + "/down.sh"
	
	def __init__(self, session, libgtools):
		gConfigWindow.__init__(self, session)
		self.__libgtools = libgtools
		ipconf = self.__readConfig()
		
		config.bp.eth_ip = NoSave(ConfigIP(default=IP2Config(ipconf[0])))
		config.bp.eth_nmask = NoSave(ConfigIP(default=IP2Config(ipconf[1])))
		config.bp.eth_bcast = NoSave(ConfigIP(default=IP2Config(ipconf[2])))
		config.bp.gateway = NoSave(ConfigIP(default=IP2Config(ipconf[3])))
		config.bp.clientnr = NoSave(ConfigInteger(default=10, limits=[1,30]))
		config.bp.createlogfile = NoSave(gConfigOnOff(default=False))
		
		if ipconf[4] is not None:
			config.bp.clientip = NoSave(ConfigIP(default=IP2Config(ipconf[4])))
		else:
			client_ip = config.bp.eth_ip.value[:]
			client_ip[3]=220
			config.bp.clientip = NoSave(ConfigIP(default=client_ip))

		list=[]
		list.append(getConfigListEntry(_("number of client:"), config.bp.clientnr))
		list.append(getConfigListEntry(_("first client ip:"), config.bp.clientip))
		list.append(CONFIGSEPERATOR())
		list.append(getConfigListEntry(_("IP") + " eth0:", config.bp.eth_ip))
		list.append(getConfigListEntry(_("Netmask") + " eth0:", config.bp.eth_nmask))
		list.append(getConfigListEntry(_("Broadcast") + " eth0:", config.bp.eth_bcast))
		list.append(getConfigListEntry(_("ip-router gateway(i.e. 192.168.0.1):"), config.bp.gateway))
		list.append(CONFIGSEPERATOR())
		list.append(getConfigListEntry(_("create logfile '%s'") %OPENVPNLOG + "?", config.bp.createlogfile))
		self.SetConfList(list)
		
	def KeyExit(self):
		self.close(False)
		
	def KeyGreen(self):
		client_nr = config.bp.clientnr.value
		client_ip = config.bp.clientip.value
		
		if client_ip[3] + client_nr -1 > 254:#-1 für Server-IP
			self.session.open(MessageBox, _("Too many clients or incorrect Start-IP!"), MessageBox.TYPE_ERROR, timeout=8)
			return

		ethip = config.bp.eth_ip.value
		nmask = config.bp.eth_nmask.value
		bcast = config.bp.eth_bcast.value
		gaway = config.bp.gateway.value
		
		try:
			file = open(self.OVPNUPSCRIPT, 'w')
			file.write("#!/bin/bash\n\nBR=$1\nDEV=$2\nMTU=$3\n\nETH=\"eth0\"\n")
			file.write("eth_ip=\"%d.%d.%d.%d\"\n" %(ethip[0],ethip[1],ethip[2],ethip[3]))
			file.write("eth_netmask=\"%d.%d.%d.%d\"\n" %(nmask[0],nmask[1],nmask[2],nmask[3]))
			file.write("eth_broadcast=\"%d.%d.%d.%d\"\n" %(bcast[0],bcast[1],bcast[2],bcast[3]))
			file.write("eth_defaultgw=\"%d.%d.%d.%d\"\n\n" %(gaway[0],gaway[1],gaway[2],gaway[3]))
			file.write("route del default\n\n")
			file.write("/usr/sbin/brctl addbr $BR\n")
			file.write("/usr/sbin/brctl addif $BR $ETH\n")
			file.write("/usr/sbin/brctl addif $BR $DEV\n\n")
			file.write("/sbin/ifconfig $DEV 0.0.0.0 mtu $MTU promisc up\n")
			file.write("/sbin/ifconfig $ETH 0.0.0.0 mtu $MTU promisc up\n")
			file.write("/sbin/ifconfig $BR $eth_ip netmask $eth_netmask broadcast $eth_broadcast\n\n")
			file.write("route add default gw $eth_defaultgw\n")
			file.close()
			chmod(self.OVPNUPSCRIPT,0755)
			
			file = open(self.OVPNDOWNSCRIPT, 'w')
			file.write("#!/bin/sh\n\nBR=$1\nDEV=$2\nETH=\"eth0\"\n\n")
			file.write("/sbin/ifconfig $BR down\n/sbin/ifconfig $DEV down\n\n")
			file.write("/usr/sbin/brctl delif $BR $DEV\n")
			file.write("/usr/sbin/brctl delif $BR $ETH\n")
			file.write("/usr/sbin/brctl delbr $BR\n")
			file.close()
			chmod(self.OVPNDOWNSCRIPT,0755)
			
			file = open(OVPNCONFIG, 'w')
			file.write("port 1194\nproto udp\nmode server\ntls-server\ndev tap0\n\n")
			file.write("up \"%s br0\"\n" %self.OVPNUPSCRIPT)
			file.write("down \"%s br0\"\n\n" %self.OVPNDOWNSCRIPT)
			file.write("server-bridge %d.%d.%d.%d %d.%d.%d.%d %d.%d.%d.%d %d.%d.%d.%d\n\n" %(client_ip[0], client_ip[1], client_ip[2], client_ip[3], nmask[0], nmask[1], nmask[2], nmask[3], client_ip[0], client_ip[1], client_ip[2], client_ip[3]+1, client_ip[0], client_ip[1], client_ip[2], client_ip[3]+1+client_nr))
			file.write("crl-verify %s/vpn-crl.pem\n\n" %OPENSSLDIR)
			file.write("ca %s/vpn-ca.pem\n" %OPENSSLDIR)
			file.write("cert %s/Server-cert.pem\n" %OPENSSLDIR)
			file.write("key %s/Server-key.pem\n" %OPENSSLDIR)
			file.write("dh %s/dh1024.pem\n\n" %OPENSSLDIR)
			file.write("push \"dhcp-option DNS %d.%d.%d.%d\"\n\n" %(gaway[0],gaway[1],gaway[2],gaway[3]))
			file.write("auth SHA1\ncipher AES-256-CBC\ncomp-lzo\n\n")
			file.write("user nobody\ngroup nogroup\n\n")
			file.write("persist-key\npersist-tun\n\n")
			file.write("verb 1\nkeepalive 10 120\n")
			file.write("status /var/run/openvpn-status.log\nscript-security 3 system\n")
			if config.bp.createlogfile.value:
				file.write("\nlog-append  %s\n" %OPENVPNLOG)
			file.close()
			
			self.close(True)
		except:
			self.session.open(MessageBox, _("Values ​​could not be written!"), MessageBox.TYPE_ERROR, timeout=8)
		
	def __readConfig(self):
		ip="192.168.0.70"
		nmask="255.255.255.0"
		bcast="192.168.0.255"
		gway="192.168.0.1"
		client_ip=None
		try:
			if pathExists(OVPNCONFIG):
				file = open(OVPNCONFIG, 'r')
				lines = file.readlines()
				file.close()
				for x in lines:
					val = x.strip()
					if len(val) and val[0]!='#' and val.startswith("server-bridge"):
						clist = val.split(' ')
						if len(clist)==5:
							client_ip=clist[1]
			
			if pathExists(self.OVPNUPSCRIPT):
				file = open(self.OVPNUPSCRIPT, 'r')
				lines = file.readlines()
				file.close()
				for x in lines:
					val = x.strip()
					if len(val) and val[0]!='#':
						clist = val.split('=')
						if len(clist)==2:
							if clist[0]=="eth_ip":
								ip=clist[1][1:-1]
							elif clist[0]=="eth_netmask":
								nmask=clist[1][1:-1]
							elif clist[0]=="eth_broadcast":
								bcast=clist[1][1:-1]
							elif clist[0]=="eth_defaultgw":
								gway=clist[1][1:-1]
			else:
				for x in self.__libgtools.getNetworkList():
					if x[NET_TYPE]==NETDEV_ETH:
						if x[NET_IP] and x[NET_MASK] and x[NET_BCAST]:
							ip=x[NET_IP]
							nmask=x[NET_MASK]
							bcast=x[NET_BCAST]
							gway=self.__libgtools.getGateway()
		except:
			print "[OVPN] error read config"
		return (ip, nmask, bcast, gway, client_ip)

class OpenVPNclient(WidgetVariableList):
	C_PID=0
	C_NAME=1
	C_SERVER=2
	C_READ=3
	C_WRITE=4
	def __init__(self, session):
		WidgetVariableList.__init__ (self, session, title=_("Openvpn-Client"), LinesProEntry=3, icon="geminiopenvpn/icon_openvpn.png", list=[])
		self.skinName = [self.__class__.__name__, "VariableListNoButton_" + self.__class__.__name__, "VariableListNoButton"]
		self.__timer = eTimer()
		self.__timer.callback.append(self.__Status)
		self.__timer.start(300)
		self.__timercount=0
		self.__binstopped=False
		self.__lastIndex=0
		dbusNotifier.append(("gvpn", self.__newevent))

	def KeyExit(self):
		del self.__timer
		dbusNotifier.remove(("gvpn", self.__newevent))
		self.close()
		cleanexit(__name__)
		
	def KeyOk(self):
		sel = self.getCurrentEntry()
		if sel and sel[2]:
			if sel[2][self.C_PID]=="0":
				self.session.openWithCallback(boundFunction(self.__connectServer,sel[2]), MessageBox, "%s ?" %_("Connect"), MessageBox.TYPE_YESNO)
			else:
				self.session.openWithCallback(boundFunction(self.__disconnectServer,sel[2]), MessageBox, "%s ?" %_("Disconnect"), MessageBox.TYPE_YESNO)
	
	def __connectServer(self, val, result):
		if result:
			cmd = "%s -f %d -n '%s'" %(GVPNEXEC,F_START_CLIENT,val[self.C_NAME])
			libgeminimain.sendDbus("gdaemon",0,cmd)
			
	def __disconnectServer(self, val, result):
		if result:
			cmd = "%s -f %d -n '%s' -k %s" %(GVPNEXEC,F_STOP_CLIENT,val[self.C_NAME], val[self.C_PID])
			libgeminimain.sendDbus("gdaemon",0,cmd)
			
	def selectionChanged(self):
		self.__lastIndex = self.GetIndex()
		
	def __newevent(self, func, value):
		#print func,"_____________________",value
		if func==F_CLIENT_STATUS:
			self.__binstopped=True
			self.SetLabel()
			self.__fillList(value)
		
	def __Status(self):
		if self.__timercount < 1:
			libgeminimain.sendDbus("gdaemon",0,"%s -f %d" %(GVPNEXEC,F_CLIENT_STATUS))
			self.__timercount=30
			self.__binstopped=False
		else:
			if self.__binstopped:
				self.__timercount -= 1
				self.SetLabel(_("read new status in %d second") %self.__timercount)
		self.__timer.start(1000,True)
		
	def __fillList(self,val):
		list=[]
		clientlist=[]
		entry=[]
		for x in val.split('\n'):
			entry.append(x)
			if len(entry)==5:
				clientlist.append(entry)
				entry=[]
		
		try:
			for x in clientlist:
				readbytes=int(x[self.C_READ])
				writebytes=int(x[self.C_WRITE])
				
				tmp = "%s: %s" %(_("Name"),x[self.C_NAME])
				if x[self.C_PID]!="0" and readbytes>0 and writebytes>0:
					tmp += " (%s - %s)" %(_("connected"),x[self.C_SERVER])
					tmp += "\n%s: %s/%s" %(_("read/write"), libgeminimain.getHumanSize(readbytes), libgeminimain.getHumanSize(writebytes))
				else:
					tmp += " (%s)" %(_("disconnected"))
				list.append((tmp, None, x))
		except:
			pass
		self.SetList(list)
		self.SetIndex(self.__lastIndex)
	