# -*- coding: utf-8 -*-
from enigma import eTimer
from Components.config import config
from Plugins.Plugin import PluginDescriptor

from Plugins.Bp.geminimain.gLocale import _
from Plugins.Bp.geminimain.lib import libgeminimain
from Plugins.Bp.geminimain.gTools import pathExists, gPluginPath
from Plugins.Bp.geminimain.plugin import dbusNotifier

import xml.etree.ElementTree as ET
import os.path
from time import strftime, localtime

WeatherPoll = None
#WEATHERBINARY="/home/root/bin/gweather"
WEATHERBINARY="/usr/bin/gweather"
WEATHERCONFIG="weather.conf"
WEATHERCACHE="/home/root/.geminicache/weather/"

def parseWeatherData(file):
	try:
		retvalue=["","","",None,None,None,None,None,None]#unit, city, date, [current], [day], [day+1]...
		entrycount=4
		if pathExists(file)==False:
			return None

		tree = ET.parse(file)
		for a in tree.getroot():
			if a.tag=="weather":
				errormess = a.attrib.get("errormessage")
				if errormess:
					return errormess.encode("utf-8", 'ignore')
				retvalue[0] = "\xc2\xb0" + a.attrib.get("degreetype").encode("utf-8", 'ignore')
				retvalue[1] = a.attrib.get("weatherlocationname").encode("utf-8", 'ignore')
				for b in a:
					if b.tag == "current":
						data = ["","","","","","","",""]#low,high,picture,text,shortday,humidity/precip,wind,date
						data[0] = b.attrib.get("temperature").encode("utf-8", 'ignore')
						data[2] = b.attrib.get("skycode").encode("utf-8", 'ignore') + ".gif"
						data[3] = b.attrib.get("skytext").encode("utf-8", 'ignore')
						data[4] = b.attrib.get("shortday").encode("utf-8", 'ignore')
						data[5] = b.attrib.get("humidity").encode("utf-8", 'ignore')
						data[6] = b.attrib.get("winddisplay").encode("utf-8", 'ignore')
						data[7] = b.attrib.get("date").encode("utf-8", 'ignore')
						retvalue[3]=data
						retvalue[2] = b.attrib.get("observationtime").encode("utf-8", 'ignore')
					if b.tag == "forecast" and entrycount<9:
						data = ["","","","","","",""]#low,high,picture,text,shortday,humidity,wind,date
						data[0] = b.attrib.get("low").encode("utf-8", 'ignore')
						data[1] = b.attrib.get("high").encode("utf-8", 'ignore')
						data[2] = b.attrib.get("skycodeday").encode("utf-8", 'ignore') + ".gif"
						data[3] = b.attrib.get("skytextday").encode("utf-8", 'ignore')
						data[4] = b.attrib.get("day").encode("utf-8", 'ignore')
						data[5] = b.attrib.get("precip").encode("utf-8", 'ignore')
						data[6] = b.attrib.get("date").encode("utf-8", 'ignore')
						
						retvalue[entrycount]=data
						entrycount += 1

		if retvalue[2]=="":
			retvalue[2] = strftime("%H:%M", localtime(os.path.getmtime(file)))

		return retvalue
	except Exception, e:
		print "[%s] <%s>"%(__name__,e)
	return None

class CWeatherPool():
	def __init__(self):
		self.InfobarTemp = [None,None,None]
		self.__Counter=60
		self.__timer = eTimer()
		self.__timer.callback.append(self.startUpdate)
		self.__timer.start(5000,True)
		dbusNotifier.append(("gweather", self.__newWevent))
		
	def __del__(self):
		dbusNotifier.remove(("gweather", self.__newWevent))

	def startUpdate(self,force=False):
		self.__Counter += 1
		if self.__Counter>60 or force==True:
			unit = libgeminimain.getStrConfig("Weather_unit", "C")
			lang = libgeminimain.getStrConfig("Weather_language", config.osd.language.value)
			cmd = "%s -l %s -v %s &" %(WEATHERBINARY,lang, unit)
			libgeminimain.sendDbus("gdaemon",0,cmd)
			self.__Counter=1
		self.__timer.start(1000*60,True)
		
	def __newWevent(self, func, value):
		if func==1:
			self.checkTemp()
		
	def getNextCheck(self):
		return self.__Counter

	def checkTemp(self):
		self.InfobarTemp=[None,None,None]
		try:
			file = WEATHERCACHE + libgeminimain.getStrConfig("Weather_lastentry", "") + ".xml"
			if pathExists(file):
				val = parseWeatherData(file)
				if val is not None and isinstance(val, list) and len(val)>8:
					if val[3] is not None:
						self.InfobarTemp[0] = val[3][0] + val[0] 
					if val[4] is not None:#Morgen
						self.InfobarTemp[1] = val[4][0] + val[0] + "   "+ val[4][1] + val[0]
					if val[5] is not None:#Morgen
						self.InfobarTemp[2] = val[5][0] + val[0] + "   "+ val[5][1] + val[0]
		except:
			print "[WEATHER] <InfobarTemp>"
		#print "____________________________________",self.InfobarTemp

#----------------------------------------------------------------

def autostart(reason, **kwargs):
	global WeatherPoll
	if reason == 0:
		if "session" in kwargs:
			WeatherPoll = CWeatherPool()
			
def main(session, **kwargs):
	from weather import gweather
	session.open(gweather)

def Plugins(**kwargs):
	return [PluginDescriptor(where = [PluginDescriptor.WHERE_AUTOSTART,PluginDescriptor.WHERE_SESSIONSTART],fnc=autostart), PluginDescriptor(name=_("Gemini-Weather"), description=_("weather plugin based on MSN-Data"), where = PluginDescriptor.WHERE_EXTENSIONSMENU, fnc=main, icon="icon_weather.png")]

#------------------------------------------------------------------
