# -*- coding: utf-8 -*-
from Plugins.Bp.geminimain.gLocale import _
from Plugins.Bp.geminimain.gTools import cleanexit, getAspectforPic, pathRemove, pathExists, gPluginPath, isHDRes, gLoadPixmap
from Plugins.Bp.geminimain.plugin import cgevent, dbusNotifier
from Plugins.Bp.geminimain.lib import libgeminimain
from Plugins.Bp.geminimain.WidgetConfig import gConfigWindow, CONFIGSEPERATOR, gConfigSelection, WidgetInput
from plugin import WeatherPoll, WEATHERBINARY, WEATHERCONFIG, WEATHERCACHE, parseWeatherData

from enigma import ePicLoad, eTimer
from Screens.Screen import Screen
from Components.Pixmap import Pixmap
from Components.Label import Label
from Components.ActionMap import ActionMap
from Components.Sources.StaticText import StaticText
from Screens.ChoiceBox import ChoiceBox
from Components.config import config, NoSave, ConfigText, getConfigListEntry
from Components.Console import Console as cConsole
from Screens.HelpMenu import HelpableScreen

from skin import loadSkin
loadSkin(gPluginPath + "geminiweather/skin.xml")

#testskin=""""""

#------------------------------------------------------------------------------

class gweather(Screen,HelpableScreen):
	ICONFOLDER=gPluginPath + "geminiweather/icons/"
	def __init__(self, session):
		#self.skin=testskin
		Screen.__init__(self, session)
		self.skinName = []
		if isHDRes == 1:
			self.skinName.append("WeatherSkin-XD")
		elif isHDRes == 0:
			self.skinName.append("WeatherSkin-HD")
		self.skinName.append("WeatherSkin")
		
		HelpableScreen.__init__(self)

		self["actions"] = ActionMap(["gActions"],
		{
			"left": self.KeyLeft,
			"right": self.KeyRight,
			"menu": self.KeyMenu,
			"ok": self.KeyOK,
			"exit": self.KeyExit
		}, -1)
		
		self["title"] = StaticText()
		self["mainline0"] = Label()
		self["mainline1"] = Label()
		self["PluginIcon"] = Pixmap()

		for x in range(5):
			self["Icon"+str(x)] = Pixmap()
			self["Text"+str(x)] = Label()
			self["SunrisePic"+str(x)] = Pixmap()
			self["SunsetPic"+str(x)] = Pixmap()
			self["SunriseTxT"+str(x)] = Label()
			self["SunsetTxT"+str(x)] = Label()
			self["Moon"+str(x)] = Pixmap()
			
		dbusNotifier.append(("gweather", self.__newWUpdate))
		
		self.picload = ePicLoad()
		self.picload.PictureData.get().append(self.__finish_decode)
		self.onLayoutFinish.append(self.__onLayoutFinish)
		
	def KeyExit(self):
		if WeatherPoll:
			WeatherPoll.checkTemp()
		dbusNotifier.remove(("gweather", self.__newWUpdate))
		del self.picload
		self.close()
		cleanexit(__name__)
		
	def __onLayoutFinish(self):

		self.setTitle(_("Gemini-Weather"))
		self["title"].setText(_("Gemini-Weather"))
		self["PluginIcon"].instance.setPixmap(gLoadPixmap("geminiweather/icon_weather.png"))
		for x in range(5):
			self["SunrisePic"+str(x)].instance.setPixmap(gLoadPixmap("geminiweather/sunrise.png"))
			self["SunsetPic"+str(x)].instance.setPixmap(gLoadPixmap("geminiweather/sunset.png"))
		sc = getAspectforPic()
		self.picload.setPara([self["Icon0"].instance.size().width(), self["Icon0"].instance.size().height(), sc[0], sc[1], 0, 1, "#ff000000"])
		
		self.helpList.append((self["actions"], "gActions", [("menu", _("Menu"))]))
		self.helpList.append((self["actions"], "gActions", [("ok", _("Update"))]))
		self.helpList.append((self["actions"], "gActions", [("left", _("prev city"))]))
		self.helpList.append((self["actions"], "gActions", [("right", _("next city"))]))
		self.helpList.append((self["actions"], "gActions", [("exit", _("Exit"))]))

		self.__usePluginIcons=cgevent.getConfig("Weather_pluginicon", 1)
		self.__lastentry = cgevent.getConfig("Weather_lastentry", "")
		self.__unit = cgevent.getConfig("Weather_unit", "C")
		self.__moonphase = cgevent.getConfig("Weather_moonphase", 1)

		self.__sunrise=[]
		self.__citylist=[]
		self.__pictureList=[]
		
		try:#Config einlesen
			file = open(WEATHERCACHE+WEATHERCONFIG, 'r')
			lines = file.readlines()
			file.close()
			for x in lines:
				val = x.strip().split(':')
				if len(val)==3:
					self.__citylist.append((val[0]+":"+val[1],val[2]))
		except Exception, e:
			print "[%s] <%s>"%(__name__,e)
		self.__newWUpdate(1,"")
		
	def __newWUpdate(self, func, value):
		if func==1:
			self.__sunrise=[]
			try:
				file = open(WEATHERCACHE + self.__lastentry + ".txt", 'r')
				lines = file.readlines()
				file.close()
				for x in lines:
					val = x.strip().split(' ')
					if len(val)==4:
						self.__sunrise.append(val)
			except Exception, e:
				print "[%s] <%s>"%(__name__,e)

			val = parseWeatherData(WEATHERCACHE + self.__lastentry + ".xml")
			if val is not None:
				if isinstance(val, str):
					self["mainline0"].setText(val)
				if isinstance(val, list) and len(val)>8:
					self.__paintPage(val)
		
	def KeyLeft(self):
		self.__getNextPrev(True)
	
	def KeyRight(self):
		self.__getNextPrev(False)
	
	def __getNextPrev(self, prev):
		index=0
		for x in self.__citylist:
			if x[0]==self.__lastentry:
				break
			index += 1

		if len(self.__citylist)>1:
			if prev==False:
				if index+1 >= len(self.__citylist):
					self.__lastentry=self.__citylist[0][0]
				else:
					self.__lastentry=self.__citylist[index+1][0]
			else:
				if index-1 < 0:
					self.__lastentry=self.__citylist[len(self.__citylist)-1][0]
				else:
					self.__lastentry=self.__citylist[index-1][0]
			cgevent.setConfig("Weather_lastentry", self.__lastentry)
		self.__newWUpdate(1,"")

	def __paintPage(self,val):
		self.__pictureList=[]

		self["mainline0"].setText(_("Weatherdata for the city '%s'") %val[1])
		self["mainline1"].setText(_("Data are from %s") %val[2])
		if val[3] is not None:
			if len(self.__sunrise)>0 and val[3][7]==self.__sunrise[0][0]:
				self["SunriseTxT0"].setText(self.__sunrise[0][1])
				self["SunsetTxT0"].setText(self.__sunrise[0][2])
				if self.__moonphase==1:
					self["Moon0"].instance.setPixmap(gLoadPixmap("geminiweather/moon%s.png" %self.__sunrise[0][3], False))
					self["Moon0"].show()
				else:
					self["Moon0"].hide()
			else:
				self["SunrisePic0"].hide()
				self["SunsetPic0"].hide()
			text = _("current") + ": " + val[3][3] + "\n"
			text += _("Temperature") + ": " + val[3][0] + val[0] + "\n"
			text += _("Humidity") + ": " + val[3][5] + "%\n"
			text += _("Wind") + ": " + val[3][6]
			self["Text0"].setText(text)
			if self.__usePluginIcons>0:
				self.__pictureList.append([0,self.ICONFOLDER + val[3][2]])
			else:
				self.__pictureList.append([0,WEATHERCACHE + val[3][2]])
		
		for x in range(5,len(val)):
			xmin4 = x-4
			if len(self.__sunrise)>=xmin4 and val[x][6]==self.__sunrise[xmin4][0]:
				self["SunriseTxT"+str(xmin4)].setText(self.__sunrise[xmin4][1])
				self["SunsetTxT"+str(xmin4)].setText(self.__sunrise[xmin4][2])
				if self.__moonphase==1:
					self["Moon"+str(xmin4)].instance.setPixmap(gLoadPixmap("geminiweather/moon%s.png" %self.__sunrise[xmin4][3], False))
					self["Moon"+str(xmin4)].show()
				else:
					self["Moon"+str(xmin4)].hide()
			else:
				self["SunrisePic"+str(xmin4)].hide()
				self["SunsetPic"+str(xmin4)].hide()
			#print val[x]
			if val[x]:
				text = val[x][4] + "\n"
				text += val[x][0] + val[0] + "   " + val[x][1] + val[0] + "\n"
				text += val[x][3] + "\n"
				text += _("Precip") + " " + val[x][5] + "%"
				self["Text"+str(x-4)].setText(text)
			if self.__usePluginIcons>0:
				self.__pictureList.append([0,self.ICONFOLDER + val[x][2]])
			else:
				self.__pictureList.append([0,WEATHERCACHE + val[x][2]])
		self.__decodePicture()

	def __decodePicture(self):
		for x in self.__pictureList:
			if x[0]==0 and x[1]!="":
				x[0]=1
				if pathExists(x[1]):
					self.picload.startDecode(x[1])
					return
				else:
					x[0]=2

	def __finish_decode(self, picInfo=""):
		for x in range(len(self.__pictureList)):
			if self.__pictureList[x][0]==1:
				self.__pictureList[x][0]=2
				if picInfo!="":
					ptr = self.picload.getData()
					if ptr != None:
						self["Icon" + str(x)].instance.setPixmap(ptr)
				self.__decodePicture()
				return

	def KeyOK(self):
		if len(self.__citylist):
			self.__UpdateNow()

	def KeyMenu(self):
		menu = []
		for x in self.__citylist:
			menu.append((x[1],x[0]))
		if len(self.__citylist):
			menu.append(("--", "--"))
		menu.append((_("City add"), "add"))
		if len(self.__citylist):
			menu.append((_("City remove"), "remove"))
		menu.append(("--", "--"))
		if len(self.__citylist) and WeatherPoll:
			menu.append((_("Update"), "update"))
		if self.__unit=="C":
			menu.append((_("unit in celsius? %s") %_("yes"), "unit"))
		else:
			menu.append((_("unit in celsius? %s") %_("no"), "unit"))
		if self.__usePluginIcons>0:
			menu.append((_("use Plugin-Icons? %s") %_("yes"), "icons"))
		else:
			menu.append((_("use Plugin-Icons? %s") %_("no"), "icons"))
		if self.__moonphase>0:
			menu.append((_("show moon? %s") %_("yes"), "moon"))
		else:
			menu.append((_("show moon? %s") %_("no"), "moon"))
		
		lang = cgevent.getConfig("Weather_language", config.osd.language.value)
		menu.append(("%s (%s '%s')" %(_("Language"), _("current"),lang), "language"))
		self.session.openWithCallback(self.__menuCallback, ChoiceBox, title="", list=menu)

	def __menuCallback(self, what=None):
		if what is None:
			return
		if what[1] == "update":
			WeatherPoll.startUpdate(True)
		elif what[1] == "icons":
			if self.__usePluginIcons>0:
				self.__usePluginIcons=0
			else:
				self.__usePluginIcons=1
			cgevent.setConfig("Weather_pluginicon", self.__usePluginIcons)
			self.__newWUpdate(1,"")
		elif what[1] == "moon":
			if self.__moonphase>0:
				self.__moonphase=0
			else:
				self.__moonphase=1
			cgevent.setConfig("Weather_moonphase", self.__moonphase)
			self.__newWUpdate(1,"")
		elif what[1] == "add":
			self.session.openWithCallback(self.__addCallback, WeatherSetup)
		elif what[1] == "unit":
			if self.__unit=="C":
				self.__unit="F"
			else:
				self.__unit="C"
			cgevent.setConfig("Weather_unit", self.__unit)
			self.__UpdateNow()
		elif what[1] == "remove":
			menu = []
			for x in self.__citylist:
				if x[0] != self.__lastentry:
					menu.append((x[1],x))
			if len(menu):
				self.session.openWithCallback(self.__removeCallback, ChoiceBox, title=_("Remove"), list=menu)
		elif what[1] == "language":
			self.session.openWithCallback(self.__language_Callback, WidgetInput, title=_("Gemini-Weather"), entryname=_("Language"), default=cgevent.getConfig("Weather_language", config.osd.language.value))
		else:
			self.__lastentry = what[1]
			cgevent.setConfig("Weather_lastentry", self.__lastentry)
			self.__UpdateNow()
			
	def __UpdateNow(self):
		if WeatherPoll:
			WeatherPoll.startUpdate(True)
			self["mainline0"].setText(_("Please wait..."))
			self["mainline1"].setText("")

	def __addCallback(self, code=None, city=None):
		if code is not None and city is not None:
			self.__citylist.append((code,city))
			self.__saveConfig()
			self.__lastentry=code
			cgevent.setConfig("Weather_lastentry", code)
			self.__UpdateNow()
			
	def __removeCallback(self, ret):
		if ret is not None:
			pathRemove(WEATHERCACHE + ret[1][0] + ".xml")
			self.__citylist.remove(ret[1])
			self.__saveConfig()
			
	def __language_Callback(self, val):
		if val is not None:
			cgevent.setConfig("Weather_language", val)
			self.__UpdateNow()
			
	def __saveConfig(self):
		try:
			file = open(WEATHERCACHE+WEATHERCONFIG, 'w')
			for x in self.__citylist:
				file.write("%s:%s\n" %(x[0],x[1]))
			file.close()
		except:
			print "[WEATHER] <write config>"

#------------------------------------------------------------------------------

class WeatherSetup(gConfigWindow):
	def __init__(self, session):
		self.Console = cConsole()
		self.__ConsoleRunning=False
		
		config.bp.city = NoSave(ConfigText(default=cgevent.getConfig("Weather_city", "Berlin"), fixed_size=False))
		list = []
		list.append(getConfigListEntry(_("City")+ ":", config.bp.city))
		gConfigWindow.__init__(self, session, list=list, greenbutton=_("Search"), icon="geminiweather/icon_weather.png")

	def KeyGreen(self):
		if self.__ConsoleRunning == False and config.bp.city.value:
			cgevent.setConfig("Weather_city", config.bp.city.value)
			self.SetLabel(_("Please wait..."))
			cmd = "%s -c %s" %(WEATHERBINARY,config.bp.city.value)
			self.Console.ePopen(cmd, self.__FinishedConsole)

	def __FinishedConsole(self, result=None, retval=None, extra_args=None):
		self.__ConsoleRunning = False
		self.SetLabel()
		if pathExists("/tmp/.cityfound"):
			file = open("/tmp/.cityfound", 'r')
			lines = file.readlines()
			file.close()
			pathRemove("/tmp/.cityfound")
			citylist=[]
			entry=[]
			for x in lines:
				val = x.strip()
				entry.append(val)
				if len(entry)==2:
					citylist.append(entry)
					entry=[]
			menu=[]
			for x in citylist:
				menu.append((x[1], x[0]))
			self.session.openWithCallback(self.__menuCallback, ChoiceBox, title=_("found"), list=menu)
		else:
			self.SetLabel(_("test fails !!!"))

	def __stopConsole(self):
		if len(self.Console.appContainers):
			for name in self.Console.appContainers.keys():
				self.Console.kill(name)
				
	def __menuCallback(self, what=None):
		if what is None:
			return
		self.__stopConsole()
		self.close(what[1],what[0])

	def KeyExit(self,ret=None):
		self.__stopConsole()
		self.close()
